#!/usr/bin/perl -w
## ######### PROJECT NAME : ##########
## 
## api_func.pm for aspiq_core.pl
## 
## ######### PROJECT DESCRIPTION : ###
## 
## This file contains api's functions for aspiq_core.pl
## 
## ###################################
## 
## Made by Boutonnet Alexandre
## Login   <aboutonnet@intellique.com>
## 
## Started on  Thu Jan 10 17:08:23 2008 Boutonnet Alexandre
## Last update Wed Oct 29 13:18:04 2008 Boutonnet Alexandre
## 
## ###################################
##

# Small hack to have static vars.
BEGIN
{
    my $object_hash = {};
    sub _Api_set_object_hash { $object_hash = shift }
    sub _Api_get_object_hash { return $object_hash }
}

sub Api_get_obj_list
{
    my $obj_path = shift;

    opendir(DIR, $obj_path) || log_critical("Unable to open $obj_path : $!");

    my @file_list = grep { /^[^\.]/ } readdir(DIR);

    closedir(DIR);

    my $object_hash = {};
    # Formatting object_hash
    foreach my $object (@file_list)
    {
	my $file_path = $obj_path.$object;

	# Testing if file is readable
	if ( ! -r $file_path )
	{
	    log_warning("Unable to read plugin : $file_path");
	    next;
	}

	# Ignoring emacs temp file
	if ($object =~ m/.*\~$/)
	{
	    log_info("$file_path is an emacs temp file, ignoring.");
	    next;
	}

	# Getting object name
	my $object_name = ($object =~ m/^(.*)_object.*/)[0];

	# If I can't get object name, I ignore
	if (!$object_name)
	{
	    log_warning("$file_path seems not to be an object, ignoring.");
	    next;
	}

	# Making hash..
	$object_hash->{$object_name} = {};
	$object_hash->{$object_name}->{'path'} = $file_path;
    }

    # Setting object hash
    _Api_set_object_hash($object_hash);
}

sub Api_data_compute
{
    my ($data_buffer) = @_;

    # Getting hash from xml
    my ($data_hash, $msg_error) = libxml::xml_to_hash($data_buffer);

    if (!$data_hash)
    {
 	output_error(4);
	log_error("An error occured during xml processing.");
 	log_debug("XML error : ".$msg_error);
	return;
    }

    # Deleting xml key for simplify process
    $data_hash = $data_hash->{'xml'};

    # Getting object hash
    my $object_hash = _Api_get_object_hash();

    # Let's go !
    foreach my $object_key (sort keys(%{$data_hash}))
    {
	if (!$object_hash->{$object_key})
	{
	    log_error("$object_key object does not exist.");

	    output_error(1, $object_key);

	    last; # or next ?
	}

	# Testing if object is instantiated
	if (!$object_hash->{$object_key}->{'instance'})
	{
	    my $error = _Api_instance_object($object_key);
	    if (ref($error) eq 'ARRAY')
	    {
		output_object_error($error->[0], $error->[1], $object_key);

		last; # or next ?
	    }
	}
	foreach my $hash (@{$data_hash->{$object_key}})
	{
	    _Api_call_method($hash, $object_key);
	}
    }
}

##### PRIVATE METHODS

sub _Api_call_method
{
    my ($data_hash, $object) = @_;

    if (ref($data_hash) ne 'HASH')
    {
	output_error(4);
 	log_error("An error occured during xml processing.");
 	log_debug("In _api_call_method, data_hash is not a hash");
	return 1;
    }

    if (!$data_hash->{'command'})
    {
	log_error("\'command\' option doesn't exists in xml.");
	output_error(2);
	return 1;
    }

    my $method = $data_hash->{'command'}[0];
    my $object_hash = _Api_get_object_hash();

    my $hash_return;


    log_info("Calling $method method of $object object.");
    # Calling method
    eval { $hash_return = $object_hash->{$object}->{'instance'}->$method($data_hash); };

    # Test if it make an error
    if ($@)
    {
	log_error($@);
	output_error(3, $method);
	return 1;
    }

    output_object_success($hash_return, $object, $method);
}

sub _Api_instance_object
{
    my $object = shift;

    my $object_hash = _Api_get_object_hash();

    log_info("Instantiation of $object object.");
    
    # Trying to instanciate the object
    eval { require($object_hash->{$object}->{'path'}); };

    # Test if it make an error
    if ($@)
    {
	log_error($@);
	return [-1, "Unable to instantiate $object object."];
    }

    $object_hash->{$object}->{'instance'} = new $object;
    # Make a test here ?

    # Init func read_only to read/write
    my $read_only = 0;

    # Read only rules tests
    $read_only = 1 if (get_config_option('read_only') eq 'no' &&
		       get_config_option('obj_read_only') =~ m/$object/);
    $read_only = 1 if (get_config_option('read_only') eq 'yes');
    
    # Initializing object.
    return ($object_hash->{$object}->{'instance'}->init($read_only));
}

1;
