#!/usr/bin/perl
## ######### PROJECT NAME : ##########
## 
## libxml.pm for aspiq
## 
## ######### PROJECT DESCRIPTION : ###
## 
## This file is the xml library
## 
## ###################################
## 
## Made by Boutonnet Alexandre
## Login   <aboutonnet@intellique.com>
## 
## Started on  Mon Jul  2 17:34:04 2007 Boutonnet Alexandre
## Last update Thu Jan 17 12:57:47 2008 Boutonnet Alexandre
##
## Verion 2.0
## 
## ###################################
##

package libxml;

# loi divine : toujours utiliser strict et warnings.
use strict;
use warnings;

# modules
use Data::Dumper;    # pour le dbogage

# pour faire du XML
use XML::LibXML;


# fonction xml -> hash : xml_to_hash ([error_code, error_msg] , \hash)
# fonction hash -> xml : hash_to_xml ([error_code, error_msg] , string)

###########################################################################
# 				fonctions
###########################################################################


sub xml_to_hash {
    my $string = shift;
    $string =~ s/>\s*</></g;

    my $parser = XML::LibXML->new();
    my $tree = eval { $parser->parse_string( $string ) };

    if($@) {
	    return (0, "$@");
    }
    else {
	    my $root = $tree->getDocumentElement;
	
	    my $ret = _boucle_read( $root );
        
        if ( ref $ret eq 'HASH' ) {
            my $hash;
            $hash->{ $root->nodeName } = $ret;
            return $hash;
        }
        elsif ( ref $ret eq 'ARRAY' ) {
            return $ret;
        }
        else {
# Erreur hash suivi par une valeur, absence de tableau
            return (0, "Unknow error");
        }

    }
}

sub hash_to_xml {
    my $hash = shift;

    return (0, "No Hash") unless ( ref $hash eq 'HASH' );
    return (0, "No xml key") unless ( exists $hash->{"xml"} );

    my $ret = _boucle_write( "xml", $hash->{"xml"} );
    return (0, "Hash invalide") unless $ret;
    return $ret;
}

###########################################################################
# 				fonctions privees
###########################################################################


sub _boucle_read {
    my $root = shift;
    my $hash;

    foreach my $child ( $root->getChildnodes ) {
        if ( $child->nodeType == 3 ) {
            return $child->data;
        }

        if ( $child->nodeType == 1 ) {
            my $text = _boucle_read($child);
	    
            if ( ref $text eq 'HASH' ) {
                my @tab;
                if ( $hash->{ $child->nodeName } ) {
                    @tab = @{ $hash->{ $child->nodeName } };
                }
                push @tab, $text;
                $hash->{ $child->nodeName } = \@tab;
            }
            elsif ( ref $text eq 'HASH' ) {
                my @tab;
                if ( $hash->{ $child->nodeName } ) {
                    @tab = @{ $hash->{ $child->nodeName } };
                }
                push @tab, $text;
                $hash->{ $child->nodeName } = \@tab;
            }
            elsif ( $text ne "" ) {
                my @tab;
                if ( $hash->{ $child->nodeName } ) {
                    @tab = @{ $hash->{ $child->nodeName } };
                }
                push @tab, $text;
                $hash->{ $child->nodeName } = \@tab;
            }
            else {
                return [0, $text ];
            }
	    
        }
        else {
            return [0, "Error Node Type ". $child->nodeType ];
        }
    }
    
    return $hash;
}

sub _boucle_write {
    my $index   = shift;
    my $param   = shift;

    my $string  = "";

    if ( ref $param eq 'HASH' ) {
        $string .= "<$index>\n";
        
        # lements fils
        foreach my $key ( sort keys %{ $param } ) {
            if ( ref $param->{$key} eq 'HASH' ) {
                my $ret1 = _boucle_write( $key, $param->{$key} );
                return undef unless $ret1;
                $string .= $ret1;
            }
            elsif ( ref $param->{$key} eq 'ARRAY' ) {
                my $ret2 = _boucle_write( $key, $param->{$key} );
                return undef unless $ret2;
                $string .= $ret2;
            }
            else {
# Erreur hash suivi par une valeur, absence de tableau
                return undef;
            }
        }

        $string .= "</$index>\n";	
    }
    elsif ( ref $param eq 'ARRAY' ) {

        # lements fils
        foreach my $value ( sort @{ $param } ) {
            if ( ref $value eq 'HASH' ) {
                my $ret3 = _boucle_write( $index, $value );
                return undef unless $ret3;
                $string .= $ret3;
            }
            elsif ( ref $value eq 'ARRAY' ) {
# Erreur tableau suivi par un tableau, absence de hash ou de valeur
                return undef;
            }
            else {
                $string .= "<$index>$value</$index>\n";
            }
        }
	
    }
    else {
# Erreur, ne peut pas arriver, on attend une hash ou un tableau
        return undef;
    }
    
    return $string;
}

1;
