#!/usr/bin/perl -w
## ######### PROJECT NAME : ##########
## 
## log_func.pm for log_func.pm
## 
## ######### PROJECT DESCRIPTION : ###
## 
## This file contains logging function for aspiq_core.pl
## 
## ###################################
## 
## Made by Boutonnet Alexandre
## Login   <aboutonnet@intellique.com>
## 
## Started on  Thu Jan 10 15:55:42 2008 Boutonnet Alexandre
## Last update Mon Jun  8 13:53:32 2009 Boutonnet Alexandre
## 
## ###################################
##

my $OBJ_LOG;

sub start_logging
{
    my $err;

    my $default_log_file = '/var/log/aspiq/core.log';
    my $log_file = get_config_option("log_file");

    if ( ! -w $log_file)
    {
	$log_file = $default_log_file;
    }
    
    # I make a new test to be sure that the
    # default one is readable..
    if ( ! -w $log_file)
    {
	print "Unable to open $log_file in writable mode.\nExiting.\n";
	exit(1);
    }
    
    ## Log level :
    # 0 : WARN, ERROR, CRITICAL
    # 1 : INFO, WARN, ERROR, CRITICAL
    # 2 : DEBUG, INFO, WARN, ERROR, CRITICAL
    my $default_log_level = 1;
    my $log_level = get_config_option("log_level");
    $log_level = $default_log_level if ($log_level < 0 || $log_level > 2);
    
    my @tab_conf = (qq{
log4perl.logger                                     = WARN, LOG-WARN
log4perl.appender.LOG-WARN                          = Log::Log4perl::Appender::File
log4perl.appender.LOG-WARN.filename                 = $log_file
log4perl.appender.LOG-WARN.layout                   = Log::Log4perl::Layout::PatternLayout
log4perl.appender.LOG-WARN.layout.ConversionPattern = %d [%p] %m %n
		    },
		    qq{
log4perl.logger                                     = INFO, LOG-INFO
log4perl.appender.LOG-INFO                          = Log::Log4perl::Appender::File
log4perl.appender.LOG-INFO.filename                 = $log_file
log4perl.appender.LOG-INFO.layout                   = Log::Log4perl::Layout::PatternLayout
log4perl.appender.LOG-INFO.layout.ConversionPattern = %d [%p] %m %n
},
		    qq{
log4perl.logger                                      = DEBUG, LOG-DEBUG
log4perl.appender.LOG-DEBUG                          = Log::Log4perl::Appender::File
log4perl.appender.LOG-DEBUG.filename                 = $log_file
log4perl.appender.LOG-DEBUG.layout                   = Log::Log4perl::Layout::PatternLayout
log4perl.appender.LOG-DEBUG.layout.ConversionPattern = %d [%p] %m %n
},
		    );

    ($err, $OBJ_LOG) = new INTLIQ::Logger();
    if ($err) {
	print $OBJ_LOG."\n";
    } else {
	my $msg;
	($err, $msg) = $OBJ_LOG->set_conf(\@tab_conf[$log_level]);
	if ($err) {
	    print "Unable to set_conf to INTLIQ::Logger : $msg\n";
	    exit(1);
	}
    }
}

# DEPRECATED function..
sub log_message
{
    my ($message, $level) = @_;

    chomp($message);

    print "(LOG) ";
    print $level." : ";
    print $message."\n";
}

sub log_error
{
    $OBJ_LOG->error(shift);
}

sub log_warning
{
    $OBJ_LOG->warn(shift);
}

sub log_info
{
    $OBJ_LOG->info(shift);
}

sub log_debug
{
    $OBJ_LOG->debug(shift);
}

sub log_critical
{
    my $msg = shift;

    print $msg."\n";
    $OBJ_LOG->critical($msg);
    exit(1);
}

1;
