#!/usr/bin/perl -w
## ######### PROJECT NAME : ##########
## 
## out_func.pm for aspiq_core.pl
## 
## ######### PROJECT DESCRIPTION : ###
## 
## this file contains all method to write datas on standard output
## 
## ###################################
## 
## Made by Boutonnet Alexandre
## Login   <aboutonnet@intellique.com>
## 
## Started on  Tue Jan 15 18:30:53 2008 Boutonnet Alexandre
## Last update Wed Oct 29 11:20:18 2008 Boutonnet Alexandre
## 
## ###################################
##

my $error_hash = {
    1 => '$$ object does not exists.',
    2 => "'command' option doesn't exists in xml.",
    3 => "method \$\$ does not exist.",
    4 => "An error occured during xml processing.",
    5 => "An error occured during xml processing.\nThis is an internal error please send an email to tech[at]intellique.com",
};

sub output_error
{
    my ($error_code, $to_substitute) = @_;

    if (!$error_hash->{$error_code})
    {
	log_critical("Hey, there is a bug in output_func.pm... I can't continue !");
    }

    my $error_msg = $error_hash->{$error_code};
    if ($to_substitute && $error_msg =~ m/\$\$/g)
    {
	$error_msg =~ s/\$\$/$to_substitute/;
    }

    _print_hash_to_xml(_make_error_xml($error_code, $error_msg));
}

sub output_object_error
{
    my ($error_code, $error_msg, $object) = @_;
    _print_hash_to_xml(_make_error_xml($error_code, $error_msg, $object));
}

sub output_object_success
{
    my ($hash, $object, $method) = @_;

    my $complete_hash = {};
    $complete_hash->{xml} = {$object => $hash};

    _print_hash_to_xml($complete_hash, $object);
}

### Private methods

sub _make_error_xml
{
    my ($error_code, $error_msg, $object) = @_;

    my $hash = {};
    if ($object)
    {
	$hash->{'xml'}->{$object}->{'error'} = [$error_code];
	$hash->{'xml'}->{$object}->{'msg'} = [$error_msg];
    }
    else
    {
	$hash->{'xml'}->{'error'} = [$error_code];
	$hash->{'xml'}->{'msg'} = [$error_msg];
    }

    return $hash;
}

sub _print_hash_to_xml
{
    my ($hash, $object) = @_;

    my ($xml, $msg_error) = libxml::hash_to_xml($hash);

    if (!$xml)
    {
	# Be carefull, in this if, I can have a infinite loop !
	if ($object)
	{
	    output_object_error(5, $error_hash->{5}, $object);
	}
	else
	{
	    output_error(5, $error_hash->{5});
	}
	log_error("An error occured during xml processing.");
	log_error("$object object generate a invalid hash.");
	log_debug("XML error (from $object object): ".$msg_error);
	return;
    }
    print $xml;
}

1;
