#!/usr/bin/perl
## ######### PROJECT NAME : ##########
## 
## aspiq_core.pl for AspIQ
## 
## ######### PROJECT DESCRIPTION : ###
## 
## This project aims to make StorIQ configuration easy.
## This file is the aspiq core.
## 
## ###################################
## 
## Made by Boutonnet Alexandre
## Login   <aboutonnet@intellique.com>
## 
## Started on  Thu Jan 10 11:14:42 2008 Boutonnet Alexandre
## Last update Tue Jun 16 16:42:09 2009 Boutonnet Alexandre
## 
## ###################################
##

## By default.
use strict;
use warnings;

## Usefull for debugging.
use Data::Dumper;

## Static variables.
my $CONF_FILE = '/etc/aspiq/aspiq.conf';
my $OBJ_PATH = '/usr/lib/aspiq/plugins/';
# my $CORE_OBJ_PATH = '/usr/lib/aspiq/lib/'; # deprecated.. cf le usr lib ..
my $SELECT_TIMER = 36;		# Time to wait for select if "-t" parameter is given
my $BUFFER_LIMIT = 524288;	# Buffer length limit.

## Global variables.
my $object_list = [];

use lib '/usr/lib/aspiq/lib/';

## Require aspiq core functions.
use conf_func;
use log_func;
use libxml;
use output_func;
use api_func;

## Require the essentials Objects.
use INTLIQ::Conf;
use INTLIQ::Logger;
use IO::Select;
use Getopt::Std;

# Managing parameters
my %parameters=();
getopts("th", \%parameters);

if (defined $parameters{h})
{
    print "Aspiq_core options :\n\n";
    print "  -h :	This help\n";
    print "  -t :	Set timer for select (".$SELECT_TIMER." seconds)\n\n";
    exit 0;
}

# I disable i/o buffering
$| = 1;

# This variable contains the select timer
my $timer = 0;
$timer = $SELECT_TIMER if (defined $parameters{t});

# Getting configuration hash.
get_config($CONF_FILE);

# Creation of the logger object
start_logging();

# Getting all objects available for the core
Api_get_obj_list($OBJ_PATH);

## Starting the big looooop containing the SELECT

my $fd;
my $read_set = new IO::Select();
my $complete_buffer = '';

# Opening standard input
open($fd, "-") || log_critical("Unable to open standard Input : $!");

# Create handle set for reading
$read_set->add($fd);

# Add the main socket to the set
while (1)
{
    # Get a set of readable handles (blocks until at least one handle is ready)
    my ($rh_set) = IO::Select->select($read_set, undef, undef, $timer);

    if (!defined($rh_set))
    {
	# Timeout !
	my @fd_array = $read_set->handles();
	foreach (@fd_array)
	{
	    $read_set->remove($_);
	    close($_);
	}
    }

    foreach my $rh (@$rh_set)
    {
	if ($rh == $fd)
	{
	    my $buf;

	    while ($buf = <$rh>)
	    {
		if($buf)
		{
		    # I got datas !
		    
		    # Buffering datas :
		    $complete_buffer .= $buf;
		    
		    # No DoS please :)
		    if (length($complete_buffer) >= $BUFFER_LIMIT)
		    {
			$complete_buffer = '';
			log_warning("Buffer max size reached, flushing current buffer..");
		    }
		    
		    # If I get the end of xml..
		    if ($buf =~ m/\<\/xml\>/)
		    {
			# I give it to my api :
			Api_data_compute($complete_buffer);
			
			# Erasing buffer
			$complete_buffer = '';
		    }
		}
	    }
	    if (!$buf)
	    { 
		# No more data to read
		$read_set->remove($rh);
		close($rh);

		# exiting..
		exit (0);
	    }
	}
	if (!$read_set->count())
	{
	    # Nothing to read : exiting
	    last;
	}
    }
}

exit (0);
