#!/usr/bin/perl
## ######### PROJECT NAME : ##########
## 
## aspiq-daemon.pl for aspiq daemon
## 
## ######### PROJECT DESCRIPTION : ###
## 
## This is the aspiq project daemon.
## 
## ###################################
## 
## Made by Boutonnet Alexandre
## Login   <aboutonnet@intellique.com>
## 
## Started on  Thu Jan 17 15:59:31 2008 Boutonnet Alexandre
## Last update Tue Jun 16 16:39:50 2009 Boutonnet Alexandre
## 
## ###################################
##

use strict;
use warnings;

## Usefull for debugging :
use Data::Dumper;

## Static variables.
my $IP_ADD_BIND = '0.0.0.0';
my $PORT = 8080;
my $PROTOCOLE = 'tcp';
my $CORE_EXEC_COMMAND = "perl /usr/sbin/aspiq-core.pl -t";
my $LOG_FILE = "/var/log/aspiq/daemon.log";

## Require the essentials packages.
use IO::Select;
use IO::Socket::INET;
use POSIX ":sys_wait_h";

## Require homebrewed objects..
use INTLIQ::Logger;

# Creation of the logger object
my ($err, $LOG) = new INTLIQ::Logger($LOG_FILE);
if ($err) {
    print $LOG."\n";
    exit(1);
}

$LOG->info("Aspiq Daemon Started with PID $$");

# Managing parameters

# I disable i/o buffering
$| = 1;

# I make tab for my child pid
my @tab_pid;

# Creating the socket
my $socket = new IO::Socket::INET(
				  LocalHost => $IP_ADD_BIND,
				  LocalPort => $PORT,
				  Proto => $PROTOCOLE,
				  Listen => 50,
				  Reuse => 1
				  ) or _critical_log("Can't bind : $@");

# Creating my select
my $read_set = new IO::Select();

# Create handle set for reading
$read_set->add($socket);

## Starting the big looooop containing the SELECT

# Add the main socket to the set
while (1)
{
    # Get a set of readable handles (blocks until at least one handle is ready)
    my ($rh_set) = IO::Select->select($read_set, undef, undef, undef);

    foreach my $rh (@$rh_set)
    {
	if ($rh == $socket)
	{
	    my $new_socket = $socket->accept();

	    my $hersockaddr = getpeername($new_socket);
	    my ($peer_port, $iaddr) = sockaddr_in($hersockaddr);
	    my $herstraddr = inet_ntoa($iaddr);
	    $LOG->info("Accepting connection from $herstraddr:$peer_port");

	    _create_child($new_socket->fileno());
 	}
    }
    if (!$read_set->count())
    {
	$LOG->info("No thing to read, exiting");
	last;
    }
}

exit(0);

# This function fork and dup socket with child i/o fd
sub _create_child
{
    my ($client) = @_;

    my $pid = fork();

    if (not defined $pid)
    {
	$LOG->error("fork : resources not avilable.");
    }
    elsif ($pid == 0)
    {
  	open(\*STDIN,  "<&$client") || _critical_log("can't dup $socket to stdin");
  	open(\*STDOUT, ">&$client") || _critical_log("can't dup $socket to stdout");
	
	exec($CORE_EXEC_COMMAND) or $LOG->error("Exec failed : $!");
	
	exit(0);
    }
    else
    {
	$LOG->info("Execution of a core instance with pid $pid");
	push @tab_pid, $pid;
	$SIG{CHLD} = \&REAPER;
    }
}

# This function waits for all terminated child in the tab
sub REAPER
{
    foreach my $pid (@tab_pid)
    {
	my $dead_pid = waitpid($pid, &WNOHANG);
	$LOG->info("Terminated core instance with pid $dead_pid") if ($dead_pid > 0);
    }
    $SIG{CHLD} = \&REAPER;
}

# This function makes the logging and die..
sub _critical_log
{
    my $txt = shift;
    
    $LOG->error($txt);

    exit(1);
}
